require 'org_lang_stats/github_api'
require 'org_lang_stats/thread_pool'

# This class contains the business logic of the project.
# It utilizes components such as the Threadpool and the
# GithubAPI to put everything together...fast
class OrgLangStats
    def self.proccess(api, org, threads, percent)
        OrgLangStats.new(api, org).proccess(percent, threads)
    end

    def initialize(api, org)
        @aggr = nil
        @repos = nil
        @api = api
        @org = org
    end

    def proccess(percent, threads)
        @repos = parallel_get_lang_stats(threads)
        @aggr = calc_aggregate_stats
        bytes_to_percent if percent
        { @org => { 'aggregate_stats' => @aggr, 'repositories' => @repos } }
    end

    def parallel_get_lang_stats(threads)
        result = {}
        pool = ThreadPool.new threads

        non_forked_repos.map do |repo|
            pool.run do
                repo_stats = @api.get_all_pages(url: repo['languages_url'])
                result.merge! Hash[repo['name'], repo_stats]
            end
        end

        pool.join_workers
        result
    end

    def non_forked_repos
        @api.get_all_pages(url: repos_url, params: { type: 'source' })
    end

    def repos_url
        "https://api.github.com/orgs/#{@org}/repos"
    end

    def calc_aggregate_stats
        @repos.values.each_with_object(Hash.new(0)) do |repo, hash|
            repo.each { |lang, bytes| hash[lang] += bytes }
        end
    end

    def bytes_to_percent
        @repos = @repos.map do |r_name, stats|
            Hash[r_name, hash_values_to_percent(stats)]
        end.inject(:merge)

        @aggr = hash_values_to_percent @aggr
    end

    def hash_values_to_percent(hash)
        total = hash.values.inject(:+)
        hash.map do |key, value|
            Hash[key, num_to_percent(value, total)]
        end.inject(:merge)
    end

    def num_to_percent(num, num_total)
        (100.0 * num / num_total).round(1).to_s + '%'
    end
end
